package com.tomwheeler.stocktrader.api;

import com.tomwheeler.stocktrader.api.events.GameListener;

/**
 * <p>
 * This interface specifies methods necessary for game play.
 * </p> 
 * 
 * @author Tom Wheeler
 */
public interface Game {
    
    /** begins a new game */
    public void startGame();
    
    /** advances the game by one day */
    public void nextDay();
    
    /** @return the number of the current day for this game */
    public int getCurrentDay();
    
    /**
     * <p>
     * Ends this game and returns the score, which is determined by the
     * total gain in the account value relative to the number of days played.
     * </p>
     * 
     * @return the player's score for this game 
     */
    public int endGame();
    
    /**
     * <p>
     * Certain operations, such as getting the current day, are not valid to be
     * called on a game that is not currently running. This method is provided so
     * that dependent classes can check whether the game is active first.
     * </p>
     * 
     * @return true if the game is currently active (i.e. has been started but
     *         not yet ended); false otherwise
     */
    public boolean isActive();

    /** 
     * @return the current profit (positive value) or loss (negative value)
     * from the beginning of the game until the present day.  This value
     * is represented as a float, so 1.30 is a 130% gain, while -0.15
     * is a 15% loss.
     */
    public float getPercentGainSinceInception();
    
    /** @return the account associated with the current game */
    public StockAccount getAccount();
    
    /** @return the stock exchange used in the current game */
    public StockExchange getExchange();
    
    /** @return the market data provider used in the current game */
    public StockDataProvider getDataProvider();

    /** registers a listener with this game so that it can be notified of changes */
    public void addGameListener(GameListener listener);
}
