package com.tomwheeler.stocktrader.api;

import java.io.Serializable;
import java.util.*;

/**
 * <p>
 * This class is a type-safe enum representing the industry types used to categorize
 * available stocks in the trading game.
 * </p> 
 * 
 * @author Tom Wheeler
 */
public final class Industry implements Serializable {

    private static final long serialVersionUID = 1L;

    public static final Industry BANKING = new Industry("Banking");
    public static final Industry ELECTRONICS = new Industry("Electronics");
    public static final Industry ENERGY = new Industry("Energy");
    public static final Industry HEALTHCARE = new Industry("Healthcare");
    public static final Industry INSURANCE = new Industry("Insurance");
    public static final Industry MANUFACTURING = new Industry("Manufacturing");
    public static final Industry MEDIA = new Industry("Media");
    public static final Industry MINING = new Industry("Mining");
    public static final Industry REAL_ESTATE = new Industry("Real Estate");
    public static final Industry RETAIL = new Industry("Retail");
    public static final Industry TELECOMMUNICATIONS = new Industry("Telecommunications");
    public static final Industry TRANSPORTATION = new Industry("Transportation");
    
    private static Industry[] VALUES = new Industry[] { BANKING, ENERGY, ELECTRONICS,
            HEALTHCARE, INSURANCE, MANUFACTURING, MEDIA, MINING, REAL_ESTATE, RETAIL,
            TELECOMMUNICATIONS, TRANSPORTATION 
            };
    
    private final String industryName;
    
    private Industry(String industryName) {
        if (industryName == null || industryName.trim().length() == 0) {
            String msg = "Industry name cannot be null, empty or consist only of spaces";
            throw new IllegalArgumentException(msg);
        }

        this.industryName = industryName;
    }
    
    /**
     * @return an immutable collection of all industries
     */
    public static Collection values() {
        return Collections.unmodifiableCollection(Arrays.asList(VALUES));
    }
    
    /**
     * <p>
     * This method returns the Industry instance corresponding to the 
     * supplied <code>industryName</code> argument, which cannot be null,
     * empty or contain only spaces.  
     * </p>
     * 
     * @param industryName the name of the industry
     * @return the (potentially null) Industry corresponding to the supplied name.
     * @throws IllegalArgumentException if the industryName was null, empty, 
     * contains only whitespace, or if no such Industry could be found.
     */
    public static Industry lookup(String industryName) {
        if (industryName == null || industryName.trim().length() == 0) {
            throw new IllegalArgumentException("The industry name was null or empty");
        }

        for (int i = 0; i < VALUES.length; i++) {
            Industry industry = VALUES[i];
            if (industry.getName().equals(industryName)) {
                return industry;
            }
        }

        throw new IllegalArgumentException("No such industry named: " + industryName);
    }

    /**
     * @return the industry's name
     */
    public String getName() {
        return industryName;
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {     
        return industryName.hashCode();
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (!(obj instanceof Industry)) {
            return false;
        }

        String thatName = ((Industry)obj).industryName;
        return industryName.equals(thatName);
    }    

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "Industry: " + industryName;
    }
}
