package com.tomwheeler.stocktrader.api;

import java.util.Collection;

/**
 * <p>
 * This interface represents services available from a stock exchange.
 * </p>
 * 
 * @author Tom Wheeler
 */
public interface StockExchange {
    
    /** 
     * @return a collection (of Stock instances) representing all stocks available 
     * for trade on this exchange.
     */
    public Collection getListedStocks();
    
    /** 
     * <p>
     * Returns a collection of all stocks listed on the exchange, filtered 
     * by the the specified market industry.
     * </p>
     * 
     * @param industry the industry of the market for which you want the stocks
     * @return a collection of Stock instances  
     */
    public Collection getListedStocks(Industry industry);
    
    /** 
     * @return the Stock instance representing the specified ticker symbol
     * @throws IllegalArgumentException if the ticker symbol is invalid 
     * (i.e. it is null or is not listed on this exchange).
     */
    public Stock getStock(String symbol) throws IllegalArgumentException;

    /**
     * <p>
     * Purchases the specified stock. 
     * </p>
     * 
     * @param acct
     *           the account to use for purchase 
     * @param stock
     *           the stock to be purchased
     * @param qty
     *           the number of shares to buy
     * @return
     *           a confirmation of the purchase (for transaction history)
     * @throws IllegalArgumentException
     *           if the number of shares is equal to or less than zero, if
     *           there is insufficient funds in the account to cover the
     *           trade, or if the ticker symbol is invalid.
     */
    public String buy(StockAccount acct, String symbol, int qty) throws IllegalArgumentException;

    /**
     * <p>
     * Sells the specified stock. 
     * </p>
     * 
     * @param acct
     *            the account with which the stock is being sold
     * @param symbol
     *            the symbol of the stock to be sold
     * @param qty
     *           the number of shares to sell
     * @return
     *           a confirmation of the sale (for transaction history)
     * @throws IllegalArgumentException
     *            if the number of shares to sell is equal to or less than zero, 
     *            if there is insufficient holdings of this stock in the account
     *            account to cover the trade, or if the ticker symbol is invalid.
     */
    public String sell(StockAccount acct, String symbol, int qty) throws IllegalArgumentException;
}
