package com.tomwheeler.stocktrader.engine;

import java.util.*;

import com.tomwheeler.stocktrader.api.*;
import com.tomwheeler.stocktrader.api.events.GameChangeEvent;
import com.tomwheeler.stocktrader.api.events.GameListener;

/**
 * <p>
 * This is a concrete implementation of the Game interface, used for
 * the stocktrader game.  It also extends the Observable class to 
 * notify observers that the current trading day has advanced.
 * </p>
 *  
 * @author Tom Wheeler
 */
public class GameImpl implements Game {
    
    // NOTE: future versions of the game may allow this to be supplied
    protected static final int DEFAULT_INITIAL_DEPOSIT = 10000;
    
    private static final short GAME_STARTED = 1;
    private static final short GAME_ENDED = 2;
    private static final short GAME_DAY_CHANGED = 3;
    
    private int currentDay = 1;
    private StockDataProvider dataProvider;
    private StockExchange exchange;
    private StockAccount account;
    private Vector gameListeners;
    private boolean hasBeenStarted = false;

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Game#startGame(int)
     */
    public void startGame() {
        hasBeenStarted = true;
        this.currentDay = 1;

        // reset everything to start with a clean slate
        dataProvider = new StockDataProviderImpl(this);
        
        exchange = new StockExchangeImpl(this);
        account = new StockAccountImpl(this, DEFAULT_INITIAL_DEPOSIT);
        
        dataProvider.performDailyInitialization();

        notifyGameListeners(new GameChangeEvent("A new game has been started"), GAME_STARTED);
    }
    
    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Game#endGame()
     */
    public int endGame() {
        int score = 0;        
        float gainPct = getPercentGainSinceInception();
        if (gainPct > 0) {
            // calculated to amplify fractional differences and because people like higher scores.
            score = (int) (getPercentGainSinceInception() * 100000000) / currentDay;    
        }
        
        hasBeenStarted = false;
        notifyGameListeners(new GameChangeEvent("The current game has ended"), GAME_ENDED);
        
        return score;        
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Game#nextDay()
     */
    public void nextDay() {
        if (! hasBeenStarted) {
            throw new IllegalStateException("Game has not been started!");
        }
        
        // have to notify the provider first so that it can prepare for requests
        currentDay++;
        dataProvider.performDailyInitialization();
        
        notifyGameListeners(new GameChangeEvent("New game day"), GAME_DAY_CHANGED);
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Game#getCurrentDay()
     */
    public int getCurrentDay() {
        if (! hasBeenStarted) {
            throw new IllegalStateException("Game has not been started!");
        }
        return currentDay;
    }
    
    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.api.Game#isStarted()
     */
    public boolean isActive() {
        return hasBeenStarted;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Game#getPercentGainSinceInception()
     */
    public float getPercentGainSinceInception() {
        if (! hasBeenStarted) {
            throw new IllegalStateException("Game has not been started!");
        }

        int stockValue = 0;
        Collection stocks = account.getStocks();
        for (Iterator iter = stocks.iterator(); iter.hasNext();) {
            Stock stock = (Stock) iter.next();
            int price = dataProvider.getPrice(stock.getSymbol(), currentDay);
            stockValue += price * account.getQuantity(stock); 
        }

        int initialDeposit = account.getInitialDepositAmount();
        int totalAssets = stockValue + account.getCashBalance();
        int profit = totalAssets - initialDeposit;
        
        if (initialDeposit == 0) {
            return 0; // avoid division by zero, just in case
        }

        return (float)profit / initialDeposit;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Game#getAccount()
     */
    public StockAccount getAccount() {
        if (! hasBeenStarted) {
            throw new IllegalStateException("Game has not been started!");
        }

        return account;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Game#getExchange()
     */
    public StockExchange getExchange() {
        if (! hasBeenStarted) {
            throw new IllegalStateException("Game has not been started!");
        }

        return exchange;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Game#getDataProvider()
     */
    public StockDataProvider getDataProvider() {
        if (! hasBeenStarted) {
            throw new IllegalStateException("Game has not been started!");
        }

        return dataProvider;
    }
    

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Game#addGameListener(com.tomwheeler.stocktrader.engine.api.event.GameListener)
     */
    public void addGameListener(GameListener listener) {
        if (gameListeners == null) {
            gameListeners = new Vector();
        }
        gameListeners.add(listener);        
    }
    
    /** convenience method to create a new event and notify registered listeners */
    private void notifyGameListeners(final GameChangeEvent event, short eventType) {
        if (gameListeners == null) {
            // no listeners registered
            return;
        }

        // TODO was getting a ConcurrentModificationException here.  I don't have time
        // to debug at the moment but should come back to it.  This hides the problem
        // for now.   UPDATE: Could it be that GameInfoComponent has a listener?
        Collection listenersCopy = (Collection)gameListeners.clone(); 
        for (Iterator iter = listenersCopy.iterator(); iter.hasNext();) {
            GameListener listener = (GameListener) iter.next();

            switch(eventType) {
                case GAME_STARTED:
                    listener.gameStarted(event);
                    break;
                case GAME_ENDED:
                    listener.gameEnded(event);
                    break;
                case GAME_DAY_CHANGED:
                    listener.dayChanged(event);
                    break;
                default:
                    String msg = "unknown event type in GameImpl.notifyListeners";
                    throw new IllegalArgumentException(msg);
            }
        }
    }
}
