package com.tomwheeler.stocktrader.engine;

import java.util.ArrayList;
import java.util.List;

import com.tomwheeler.stocktrader.api.Industry;

/**
 * <p>
 * This class generates news stories that determine what industries will be affected,
 * and whether it will have a positive or negative change on stock prices.
 * I have tried intentionally tried not to make this too specific to the United States
 * market, so it would have interest to people in other countries. 
 * </p> 
 * 
 * @author Tom Wheeler
 */
public class MarketNewsGenerator {
    
    /* SPOILER WARNING: All the mysteries of the market are revealed herein!  */

    private boolean isBearMarket;
    
    private static List bearMarketStories = createBearMarketStories();
    private static List bullMarketStories = createBullMarketStories();

    MarketNewsGenerator(boolean isBearMarket) {
        this.isBearMarket = isBearMarket;
    }
    
    /**
     * <p>
     * This method returns news items that are used to influence market activity
     * for today.
     * </p>
     *  
     * <p>
     * The current implementation only ever returns one story, but the return value 
     * is a List to allow future versions to return multiple stories without changing 
     * the method signature.  Also, though this chooses stories randomly from a static
     * list, I chose to make this an instance method since future versions should
     * probably track state so that they don't reuse previous stories too soon.
     * </p>
     * 
     * @return a list of NewsStory items.  
     */
    public List getStories() {
        List retval = new ArrayList(1);
        
        MarketNewsStory story = null;
        if (isBearMarket) {
            int idx = (int) (Math.random() * bearMarketStories.size());
            retval.add(bearMarketStories.get(idx));
        } else {
            int idx = (int) (Math.random() * bullMarketStories.size());
            retval.add(bullMarketStories.get(idx));
        }

        return retval;
    }

    /** 
     * <p>
     * This method populates the internal list of stories used during a bull 
     * market.  It's intended to be called only once: upon initialization of
     * the class.  Note that the game implementation (like the actual stock 
     * market) favors growth over time.  This is manifested in the fact that
     * bull market stories are more likely to be requested than bear market
     * stories, so there should be a larger list of bull market stories from 
     * which to choose.
     * </p>
     * 
     * @return the list of stories used during a bull market
     */
    private static List createBullMarketStories() {
        List stories = new ArrayList();
        
        String txt = "Petroleum refining overcapacity caused fuel prices to drop by 15%.";
        MarketNewsStory lowerFuelStory = new MarketNewsStory(txt); 
        // less money spent on fuel, more for profit or reinvestment
        lowerFuelStory.setIndustryImpact(Industry.TRANSPORTATION, 0.09F);
        // cheaper energy decreases cost of production
        lowerFuelStory.setIndustryImpact(Industry.MANUFACTURING, 0.08F);
        // people spending less on fuel, so they have more for discretionary purchases
        lowerFuelStory.setIndustryImpact(Industry.RETAIL, 0.07F);
        // less revenue for sellers/distributors of petroleum-based fuel
        lowerFuelStory.setIndustryImpact(Industry.ENERGY, -0.06F);
        stories.add(lowerFuelStory);        
        
        txt = "SuperMegaTech Magazine reports that the number of homes "
            + "with broadband internet access rose to 78%.";
        MarketNewsStory bbAccessStory = new MarketNewsStory(txt);
        // selling more routers, computers, etc.
        bbAccessStory.setIndustryImpact(Industry.ELECTRONICS, 0.07F);
        // selling more internet access
        bbAccessStory.setIndustryImpact(Industry.TELECOMMUNICATIONS, 0.07F);
        // selling more wires and cables
        bbAccessStory.setIndustryImpact(Industry.MINING, 0.03F);
        // selling more dreck
        bbAccessStory.setIndustryImpact(Industry.MEDIA, 0.06F);
        // more people use audioconferencing/videoconferencing and travel less
        bbAccessStory.setIndustryImpact(Industry.TRANSPORTATION, -0.02F);
        bbAccessStory.setIndustryImpact(Industry.ENERGY, -0.02F);
        stories.add(bbAccessStory);
        
        txt = "MegaMedia Corporation today announced a merger with Omnipresent Advertising Group.";
        MarketNewsStory mediaMergerStory = new MarketNewsStory(txt);
        // consolidation is seen as positive in this case ("synergies", ha ha)
        mediaMergerStory.setIndustryImpact(Industry.MEDIA, 0.09F);
        // "content" prices are expected to go down (i.e. a wishful delusion)
        mediaMergerStory.setIndustryImpact(Industry.TELECOMMUNICATIONS, 0.04F);
        mediaMergerStory.setIndustryImpact(Industry.RETAIL, 0.03F);
        stories.add(mediaMergerStory);
        
        txt = "The National Copper Council reported a 14% drop in copper prices.";
        MarketNewsStory cheapCopperStory = new MarketNewsStory(txt);
        // wires, cables and tubing are cheaper now
        cheapCopperStory.setIndustryImpact(Industry.TELECOMMUNICATIONS, 0.05F);
        cheapCopperStory.setIndustryImpact(Industry.MANUFACTURING, 0.04F);
        cheapCopperStory.setIndustryImpact(Industry.ELECTRONICS, 0.03F);
        // less revenue for the copper miners
        cheapCopperStory.setIndustryImpact(Industry.MINING, -0.03F);
        stories.add(cheapCopperStory);
        
        txt = "The Department of Labor today released statistics that "
            + "show a 12% drop in on-the-job injuries.";
        MarketNewsStory fewerInjuriesStory = new MarketNewsStory(txt);
        // fewer claims to pay by the insurance companies
        fewerInjuriesStory.setIndustryImpact(Industry.INSURANCE, 0.08F);
        // lower premiums for industries that rely on a lot of manual labor
        fewerInjuriesStory.setIndustryImpact(Industry.MINING, 0.05f);
        fewerInjuriesStory.setIndustryImpact(Industry.MANUFACTURING, 0.04F);        
        fewerInjuriesStory.setIndustryImpact(Industry.TRANSPORTATION, 0.02F);
        // fewer injuries mean fewer patients 
        fewerInjuriesStory.setIndustryImpact(Industry.HEALTHCARE, -0.03F);
        stories.add(fewerInjuriesStory);

        txt = "The legislature has just passed tort reform limiting medical malpractice claims.";
        MarketNewsStory tortReformStory = new MarketNewsStory(txt);
        tortReformStory.setIndustryImpact(Industry.INSURANCE, 0.12F);
        tortReformStory.setIndustryImpact(Industry.HEALTHCARE, 0.09F);        
        stories.add(tortReformStory);
        
        txt = "The National Bankers Associated reported that increased use of the Internet "
            + "for customer self-service has reduced banks' labor costs by 12%.";
        MarketNewsStory bankLaborReductionStory = new MarketNewsStory(txt);
        bankLaborReductionStory.setIndustryImpact(Industry.BANKING, 0.07F);
        bankLaborReductionStory.setIndustryImpact(Industry.TELECOMMUNICATIONS, 0.03F);
        stories.add(bankLaborReductionStory);
        
        txt = "ChemE Corporation today announced a new process for extracting "
            + "petroleum from shale.  This is expected to reduce energy company "
            + "processing costs by as much as 17%.";
        MarketNewsStory shaleExtractionStory = new MarketNewsStory(txt);
        // In the interest of parity, the energy industry needs a break; assume that 
        // they're keeping all the money for themselves and fuel prices won't drop
        shaleExtractionStory.setIndustryImpact(Industry.ENERGY, 0.07F);
        stories.add(shaleExtractionStory);
        
        txt = "The government announced today that it will vastly expand tax incentives "
            + "for the redevelopment of abandonded industrial property";
        MarketNewsStory indRedevelStory = new MarketNewsStory(txt);
        indRedevelStory.setIndustryImpact(Industry.REAL_ESTATE, 0.09F);
        stories.add(indRedevelStory);

        txt = "The central bank today increased interest rates by 25 basis points.";
        MarketNewsStory lowerRateStory = new MarketNewsStory(txt);
        // using credit is now less expensive
        lowerRateStory.setIndustryImpact(Industry.REAL_ESTATE, 0.08F);
        lowerRateStory.setIndustryImpact(Industry.RETAIL, 0.07F);
        // will invest in R&D and new equipment
        lowerRateStory.setIndustryImpact(Industry.ELECTRONICS, 0.02F);
        // banks typically lose when rates fall
        lowerRateStory.setIndustryImpact(Industry.BANKING, -0.02F);
        stories.add(lowerRateStory);

        return stories;
    }
    
    /** 
     * <p>
     * This method populates the internal list of stories used during a bull market.
     * It's intended to be called only once: upon initialization of the class.
     * For reasons documented in the <code>createBearMarketStories</code> method, 
     * there should be more bull market stories than bear market stories.
     * </p>
     * 
     * @return the list of stories used during a bear market
     */
    private static List createBearMarketStories() {
        List stories = new ArrayList();
        
        String txt = "The central bank today increased interest rates by 50 basis points.";
        MarketNewsStory higherRateStory = new MarketNewsStory(txt);
        // using credit now more expensive
        higherRateStory.setIndustryImpact(Industry.REAL_ESTATE, -0.09F);
        higherRateStory.setIndustryImpact(Industry.RETAIL, -0.06F);
        // postponing reinvestment in R&D and new equipment
        higherRateStory.setIndustryImpact(Industry.TELECOMMUNICATIONS, -0.02F);
        higherRateStory.setIndustryImpact(Industry.ELECTRONICS, -0.02F);
        higherRateStory.setIndustryImpact(Industry.MANUFACTURING, -0.02F);
        higherRateStory.setIndustryImpact(Industry.MINING, -0.01F);
        // banks typically profits from higher rates
        higherRateStory.setIndustryImpact(Industry.BANKING, 0.05F);
        stories.add(higherRateStory);
        
        txt = "The Labor Department stated today that the unemployment rate rose to 7%.";
        MarketNewsStory unempRiseStory = new MarketNewsStory(txt);
        // less money for discretionary purchases
        unempRiseStory.setIndustryImpact(Industry.RETAIL, -0.04F);
        // less money for deposit
        unempRiseStory.setIndustryImpact(Industry.BANKING, -0.03F);
        // cheaper labor
        unempRiseStory.setIndustryImpact(Industry.MANUFACTURING, 0.03F);
        stories.add(unempRiseStory);
        
        txt = "A typhoon has hit the the eastern coast, causing significant property damage.";
        MarketNewsStory hurricaneStory = new MarketNewsStory(txt);
        // they may have to pay claims
        hurricaneStory.setIndustryImpact(Industry.INSURANCE, -0.06F);
        // loan defaults rise in the affected area (offset by new loans granted for rebuilding)
        hurricaneStory.setIndustryImpact(Industry.BANKING, -0.03F);
        // there are seaports there so products and fuel can't come in
        hurricaneStory.setIndustryImpact(Industry.ENERGY, -0.02F);
        hurricaneStory.setIndustryImpact(Industry.TRANSPORTATION, -0.02F);
        // damage to phone and data lines has to be repaired
        hurricaneStory.setIndustryImpact(Industry.TELECOMMUNICATIONS, -0.02F);
        stories.add(hurricaneStory);
        
        txt = "The National Health Department reported a 14% increase in medical malpractice suits.";
        MarketNewsStory malpracticeRiseStory = new MarketNewsStory(txt);
        // they have to pay for claims
        malpracticeRiseStory.setIndustryImpact(Industry.INSURANCE, -0.05F);
        // industry is skiddish about this sort of thing
        malpracticeRiseStory.setIndustryImpact(Industry.HEALTHCARE, -0.04F);
        stories.add(malpracticeRiseStory);
        
        txt = "The National Paper Institute described the price of paper as 'skyrocketing'";
        MarketNewsStory costlyPaperStory = new MarketNewsStory(txt);
        // newsprint and magazine stock is more expensive
        costlyPaperStory.setIndustryImpact(Industry.MEDIA, -0.07F);
        // bank statements and other records are more expensive to print
        costlyPaperStory.setIndustryImpact(Industry.BANKING, -0.02F);
        stories.add(costlyPaperStory);

        txt = "Global Geology Prospecters today announced that their expedition in  "
            + "North Fooland has failed to locate new petroleum deposits.";
        MarketNewsStory failedOilExpeditionStory = new MarketNewsStory(txt);
        // energy supplies < demand means increased production/transportation costs
        failedOilExpeditionStory.setIndustryImpact(Industry.TRANSPORTATION, -0.05F);
        failedOilExpeditionStory.setIndustryImpact(Industry.MANUFACTURING, -0.03F);         
        // people spending more on fuel, so they have less for discretionary purchases
        failedOilExpeditionStory.setIndustryImpact(Industry.RETAIL, -0.02F);
        // oil miners won't be getting this job
        failedOilExpeditionStory.setIndustryImpact(Industry.MINING, -0.02F);
        // energy supplies < demand means higher prices and more revenue for oil companies
        failedOilExpeditionStory.setIndustryImpact(Industry.ENERGY, 0.03F);
        stories.add(failedOilExpeditionStory);

        txt = "Recent polls indicate that audiences are bored with movie remakes, reality "
            + "television shows and uninspired music.";
        MarketNewsStory boredAudienceStory = new MarketNewsStory(txt);
        // justice is done...
        boredAudienceStory.setIndustryImpact(Industry.MEDIA, -0.09F);
        // selling fewer CD players, televisions, etc.
        boredAudienceStory.setIndustryImpact(Industry.ELECTRONICS, -0.03F);
        // selling fewer CDs, DVDs, etc.
        boredAudienceStory.setIndustryImpact(Industry.RETAIL, -0.02F);
        stories.add(boredAudienceStory);
        
        txt = "The National Communications Agency today announced that they are pursuing "
            + "a plan for increased regulation of telephone companies.";
        MarketNewsStory telecomRegStory = new MarketNewsStory(txt);
        boredAudienceStory.setIndustryImpact(Industry.TELECOMMUNICATIONS, -0.09F);
        stories.add(telecomRegStory);
        
        return stories;
    }
}
