package com.tomwheeler.stocktrader.engine;

import java.util.*;

import com.tomwheeler.stocktrader.api.Industry;
import com.tomwheeler.stocktrader.api.NewsStory;

/**
 * <p>
 * This class represents a news story that is relevant to the stock market, as well as
 * its positive and negative effects.  It is intended for use by classes that implement
 * the com.tomwheeler.stocktrader.engine.api.StockDataProvider interface.  In the current 
 * version of the game, only the story text is intended for display to the user.  To 
 * find out the industries affected, they should think about the story and how might affect
 * the market (or alternatively, an enterprising player could read the source code).
 * </p>
 *  
 * @author Tom Wheeler
 */
public class MarketNewsStory implements NewsStory {
    
    private String storyText;    
    private Map positive; // key = industry, value = Float
    private Map negative; // key = industry, value = Float
    
    /** 
     * <p>
     * Creates a new story with the specified text.  After creating the story, you
     * should call <code>setIndustryImpact(Industry, float)</code> to set the effects
     * on industries of the market that will rise or fall based on this news.
     * </p>
     * 
     * @param storyText the text of the story, which explains what happened
     */
    MarketNewsStory(String storyText) {
        this.storyText = storyText;
        positive = new HashMap();
        negative = new HashMap();
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.api.NewsStory#getStoryText()
     */
    public String getStoryText() {
        return storyText;
    }

    /**
     * <p>
     * Sets the effect (rise or fall) on a industry of the market that is affected by 
     * this news story.  If the change is a positive value, then stocks in this 
     * industry will rise, while a negative value will cause stocks in this industry 
     * to fall.  Note that <code>percent</code> argument is used as a baseline, 
     * and prices for individual stocks within that industry will vary slightly
     * based on a random number so that game play is slightly less predictable. 
     * </p>
     * 
     * @param industry the market industry that is affected by the news
     * @param percent the percentage used as a baseline for the rise or fall 
     * of stock prices in this industry.  For example, a <code>percent</code>
     * value of <code>0.04f</code> means that stocks will rise approximately
     * four percent, while a <code>percent</code> value of <code>-0.12f</code> 
     * means that stocks will fall approximately twelve percent.
     */
    void setIndustryImpact(Industry industry, float percent) {
        if (industry == null || percent == 0) {
            // would be useless to store this, so we'll quietly ignore it
            return;
        }

        if (percent > 0) {
            positive.put(industry, new Float(percent));    
        } else {
            negative.put(industry, new Float(percent));    
        }        
    }

    /** @return a Collection of industries that will benefit from this news story */
    Collection getPositivelyAffectedIndustries() {
        return Collections.unmodifiableCollection(positive.keySet()); 
    }
    
    /** @return a Collection of industries that will <b>not</b> benefit from this news story */
    Collection getNegativelyAffectedIndustries() {
        return Collections.unmodifiableCollection(negative.keySet()); 
    }
    
    /**
     * <p>
     * This method returns the percent change (positive for gain, negative for
     * loss) by which the specified industry is affected by this news story.
     * </p>
     * 
     * @param industry the Industry for which you want to determine the change
     * @return the percentage change for this industry; will return zero if
     * this industry is not affected by the news
     */
    float getChangePercent(Industry industry) {
        float netChange = 0f;

        // don't assume that this story has either one positive or one negative effect 
        Float negativeChange = (Float)negative.get(industry);
        if (negativeChange != null) {
            netChange += negativeChange.floatValue(); // stored as a signed value
        }

        Float positiveChange = (Float)positive.get(industry);
        if (positiveChange != null) {
            netChange += positiveChange.floatValue();
        }

        return netChange; 
    }    
}
