package com.tomwheeler.stocktrader.engine;

import java.text.NumberFormat;
import java.util.*;

import com.tomwheeler.stocktrader.api.*;
import com.tomwheeler.stocktrader.api.events.*;

/**
 * <p>
 * This is a concrete implementation of the StockDataProvider interface, used for
 * the stocktrader game.
 * </p>
 *  
 * @author Tom Wheeler
 */
public class StockDataProviderImpl implements StockDataProvider {

    private Game game;
    private static NumberFormat currencyFormatter;
    private Vector stockPriceListeners;
    private Vector newsStoryListeners;
    
    // key=day # (Integer), val=List of stories (String)
    private Map newsItemArchive; 

    // key=day # (Integer), val=List of HistoricalData
    private Map historicalData; 
    
    public StockDataProviderImpl(Game game)  {
        currencyFormatter = NumberFormat.getCurrencyInstance();
        
        this.game = game;
        game.addGameListener(new GameProviderListener());
        
        newsItemArchive = new HashMap();
        historicalData = new HashMap();
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockDataProvider#getPrice(java.lang.String, int)
     */
    public int getPrice(String symbol, int dayNumber) throws IllegalArgumentException {
        HistoricalData data = getDataForSymbol(symbol, dayNumber);
        
        int price = 0;
        if (data != null) {
            price = data.getPrice();
        }
        return price;
    }

    /** returns HistoricalData for the specified symbol and day, or null if no match is found */
    private HistoricalData getDataForSymbol(String symbol, int dayNumber) throws IllegalArgumentException {
        game.getExchange().getStock(symbol); // verifies valid symbol and throws IAE if invalid
        if (dayNumber < 1 || dayNumber > game.getCurrentDay()) {
            throw new IllegalArgumentException("date for historical data is out of range");
        }

        List allDataForDay = (List)historicalData.get(new Integer(dayNumber));
        for (Iterator iter = allDataForDay.iterator(); iter.hasNext();) {
            HistoricalData data = (HistoricalData) iter.next();
            if (data.getSymbol().equals(symbol)) {
                return data;
            }
        }
        
        return null;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockDataProvider#getNewsItems(int)
     */
    public Collection getNewsItems(int dayNumber) {
        List newsItems = new ArrayList();
        List newsForDay = (List)newsItemArchive.get(new Integer(dayNumber));
        for (Iterator iter = newsForDay.iterator(); iter.hasNext();) {
            String story = (String) iter.next();
            newsItems.add(story);            
        }
        return Collections.unmodifiableCollection(newsItems);
    }

    /** populates news items, plus today's stock prices */
    public void performDailyInitialization() {
        // TODO: simplify this method
        
        // we want growth over time with some periodic downturns, so a bear market will
        // happen about three out eight times, meaning that more bad things will happen  
        // than good things. The converse is true for a bull market.
        boolean isBearMarket = (Math.random() <= 0.375);
        int currentDayNum = game.getCurrentDay();

        // create the current news for today; this influences stock prices
        MarketNewsGenerator generator = new MarketNewsGenerator(isBearMarket);
        List stories = generator.getStories();

        Collection losingIndustries, winningIndustries;
        Map changeByIndustry = new HashMap();
        for (Iterator storyIter = stories.iterator(); storyIter.hasNext();) {
            MarketNewsStory story = (MarketNewsStory) storyIter.next();
            
            losingIndustries = story.getNegativelyAffectedIndustries();            
            for (Iterator lsIter = losingIndustries.iterator(); lsIter.hasNext();) {
                Industry ls = (Industry) lsIter.next();
                Float industryChangePercent = new Float(story.getChangePercent(ls));
                changeByIndustry.put(ls, industryChangePercent);
            }
            
            winningIndustries = story.getPositivelyAffectedIndustries();
            for (Iterator wsIter = winningIndustries.iterator(); wsIter.hasNext();) {
                Industry ws = (Industry) wsIter.next();
                Float industryChangePercent = new Float(story.getChangePercent(ws));

                // theoretically possible for a industry to have both a loss and a gain on same day
                Float todayIndustryLoss = (Float)changeByIndustry.get(ws);
                if (todayIndustryLoss != null) {
                    float netChange = industryChangePercent.floatValue() + todayIndustryLoss.floatValue(); 
                    industryChangePercent = new Float(netChange);
                }
                changeByIndustry.put(ws, industryChangePercent);
            }

            notifyNewsStoryListeners(new NewsStoryEvent("new story", story));
        }

        // save today's news for posterity
        newsItemArchive.put(new Integer(game.getCurrentDay()), stories);

        // increase/decrease all stocks slightly based on bull/bear market
        List todaysDataList = new ArrayList();
        StockExchange exchange = game.getExchange();
        Collection allStocks = exchange.getListedStocks();
        for (Iterator stockIter = allStocks.iterator(); stockIter.hasNext();) {
            Stock stock = (Stock) stockIter.next();
            String symbol = stock.getSymbol();
            Industry industry = stock.getIndustry();

            // on the first day, we have to make up all the prices
            int price = (int) (Math.random() * 75) + 10;

            // each subsequent day, we base it on previous day's price
            // stock performance tends to increase, otherwise you'd lose all the time.
            if (currentDayNum > 1) {
                HistoricalData lastData = getDataForSymbol(symbol, currentDayNum - 1);
                if (isBearMarket) {
                    int loss = (int) Math.random() * 3;
                    price = lastData.getPrice() - loss;
                } else {
                    int gain = (int) Math.random() * 7;
                    price = lastData.getPrice() + gain;
                }

                // now adjust some more if the stock's industry was in the news today
                Float industryChange = (Float)changeByIndustry.get(industry);
                if (industryChange != null) {
                    // individual stock prices track the industry, but vary slightly
                    float stockChange = industryChange.floatValue();
                    if (Math.random() < 0.500) {
                        stockChange -= Math.random() * 0.02;                        
                    } else {
                        stockChange += Math.random() * 0.03;   
                    }                    

                    price = (int)(price * (1 + stockChange));
                }                
            }

            todaysDataList.add(new HistoricalData(symbol, price));            
            // impl detail: it's inefficient to keep updating the map like this, but it's
            /// better for callers since it is always up-to-date
            historicalData.put(new Integer(currentDayNum), todaysDataList);

            StockPriceChangeEvent event = new StockPriceChangeEvent("price update", symbol);
            notifyStockPriceListeners(event);
        }
        historicalData.put(new Integer(currentDayNum), todaysDataList);
        
        // NOTE: listeners would not be hooked up on the first day of the game, but
        // it's not a problem because the stock prices on the first day are a baseline
    }
    
    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.api.StockDataProvider#addNewsStoryListener(com.tomwheeler.stocktrader.api.events.NewsStoryListener)
     */
    public void addNewsStoryListener(NewsStoryListener listener) {
        if (newsStoryListeners == null) {
            newsStoryListeners = new Vector();
        }
        newsStoryListeners.add(listener);        
    }
    
    
    /** convenience method to notify registered news story listeners */
    private void notifyNewsStoryListeners(NewsStoryEvent event) {
        if (newsStoryListeners == null) {
            // no listeners registered
            return;
        }
        
        for (Iterator iter = newsStoryListeners.iterator(); iter.hasNext();) {
            NewsStoryListener listener = (NewsStoryListener) iter.next();
            listener.newsStoryPublished(event);
        }
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.api.StockDataProvider#addStockPriceListener(com.tomwheeler.stocktrader.api.events.StockPriceListener)
     */
    public void addStockPriceListener(StockPriceListener listener) {
        if (stockPriceListeners == null) {
            stockPriceListeners = new Vector();
        }
        stockPriceListeners.add(listener);        
    }
    
    /** convenience method to notify registered stock price listeners */
    private void notifyStockPriceListeners(StockPriceChangeEvent event) {
        if (stockPriceListeners == null) {
            // no listeners registered
            return;
        }
        
        for (Iterator iter = stockPriceListeners.iterator(); iter.hasNext();) {
            StockPriceListener listener = (StockPriceListener) iter.next();
            listener.stockPriceUpdated(event);
        }
    }
    
    // so we can trigger generation of market stories and new stock prices
    private class GameProviderListener implements GameListener {

        public void gameStarted(GameChangeEvent event) { }

        public void gameEnded(GameChangeEvent event) { 
            newsItemArchive.clear();
            historicalData.clear();
        }
        
        public void dayChanged(GameChangeEvent event) { }
    }

    /** used internally for storing historical data about the stocks */
    private class HistoricalData {
        final private String symbol;
        final private int price;
        
        HistoricalData(String symbol, int price) {
            this.symbol = symbol;
            this.price = price;
        }
        
        String getSymbol() { return symbol; }
        int getPrice() { return price; }
    }

}
